@@ -1,6 +1,6 @@
 /* header file for eBook-speaker
  *
- * Copyright (C) 2017 J. Lemmens
+ * Copyright (C)2020 J. Lemmens
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -97,20 +97,19 @@ typedef struct Misc
    int total_phrases_in_opf, total_phrases_in_ncx;
    int use_cuneiform, pause_resume_playing, break_phrase;
    int ncx_failed, opf_failed, verbose;
-   int start_arg_is_a_dir;
-   long min_vol, max_vol, volume;
+   int start_arg_is_a_dir, no_smil_items, update_progress;
    float speed;
    htmlDocPtr doc;
    xmlTextReaderPtr reader;
    pid_t player_pid, main_pid;
-   char *pulseaudio_device;
+   int pulseaudio_device;
    char ncc_html[MAX_STR], ncc_totalTime[MAX_STR];
    char daisy_version[MAX_STR], daisy_title[MAX_STR], daisy_language[MAX_STR];
    char tag[MAX_TAG], *label, ocr_language[5];
    char bookmark_title[MAX_STR];
    char cd_dev[MAX_STR], tts[15][MAX_STR];
    char option_t[MAX_STR];
-   char opf_name[MAX_STR], ncx_name[MAX_STR], copyright[MAX_STR], *option_d;
+   char opf_name[MAX_STR], ncx_name[MAX_STR], copyright[MAX_STR], option_d[5];
    char eBook_speaker_txt[MAX_STR + 1], tmp_wav[MAX_STR + 1];
    char cmd[MAX_CMD + 1], str[MAX_STR + 1];
    char *src_dir, *tmp_dir, *daisy_mp;
@@ -150,7 +149,6 @@ extern int get_page_number_2 (misc_t *, my_attribute_t *, daisy_t *, char *);
 extern int get_page_number_3 (misc_t *, my_attribute_t *);        
 extern void playfile (misc_t *, char *, char *, char *, char *, char *);
 extern void fill_page_numbers (misc_t *, daisy_t *, my_attribute_t *);
-extern void parse_page_number (misc_t *, my_attribute_t *, xmlTextReaderPtr);
 extern void check_pulseaudio_device (misc_t *, daisy_t *);
 extern void select_next_output_device (misc_t *, daisy_t *);
 extern void go_to_page_number (misc_t *, my_attribute_t *, daisy_t *);
@@ -179,7 +177,8 @@ extern void create_epub (misc_t *, my_attribute_t *, daisy_t *, char *, int);
 extern void play_epub (misc_t *, my_attribute_t *, daisy_t *, char *);
 extern char *ascii2html (misc_t *, char *);
 extern void write_ascii (misc_t *, my_attribute_t *, daisy_t *, int, char *);
-extern void get_realpath_name (char *, char *, char *);
+extern void get_real_pathname (char *, char *, char *);
 extern void parse_opf (misc_t *, my_attribute_t *, daisy_t *);
 extern void reset_term_signal_handlers_after_fork (void);
 extern void parse_smil_opf (misc_t *, my_attribute_t *, daisy_t *, int);
+extern void show_progress (misc_t *, daisy_t *);
